@@ -4,6 +4,8 @@
  * Copyright (C) 2008 Openismus GmbH <http://openismus.com/>
  * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
  *
+ * SPDX-License-Identifier: MIT
+ *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
  * files (the "Software"), to deal in the Software without
@@ -188,9 +190,12 @@ DBusPyServer_new_connection_cb(DBusServer *server,
                                void *data UNUSED)
 {
     PyGILState_STATE gil = PyGILState_Ensure();
+    PyObject *et, *ev, *tb;
     PyObject *self = NULL;
     PyObject *method = NULL;
 
+    PyErr_Fetch(&et, &ev, &tb);
+
     self = DBusPyServer_ExistingFromDBusServer(server);
     if (!self) goto out;
     TRACE(self);
@@ -222,12 +227,12 @@ DBusPyServer_new_connection_cb(DBusServer *server,
     }
 
 out:
-    Py_CLEAR(method);
-    Py_CLEAR(self);
-
     if (PyErr_Occurred())
-        PyErr_Print();
+        PyErr_WriteUnraisable(method);
 
+    Py_CLEAR(self);
+    Py_CLEAR(method);
+    PyErr_Restore(et, ev, tb);
     PyGILState_Release(gil);
 }
 
@@ -533,7 +538,7 @@ Server_get_is_connected (Server *self, PyObject *args UNUSED)
 /* Server type object =============================================== */
 
 struct PyMethodDef DBusPyServer_tp_methods[] = {
-#define ENTRY(name, flags) {#name, (PyCFunction)Server_##name, flags, Server_##name##__doc__}
+#define ENTRY(name, flags) {#name, (PyCFunction) (void (*)(void))Server_##name, flags, Server_##name##__doc__}
     ENTRY(disconnect,       METH_NOARGS),
     ENTRY(get_address,      METH_NOARGS),
     ENTRY(get_id,           METH_NOARGS),
