@@ -3,6 +3,8 @@
 # Copyright (C) 2004 Red Hat Inc. <http://www.redhat.com/>
 # Copyright (C) 2005-2007 Collabora Ltd. <http://www.collabora.co.uk/>
 #
+# SPDX-License-Identifier: MIT
+#
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
 # files (the "Software"), to deal in the Software without
@@ -39,6 +41,7 @@ except ImportError:
 import dbus
 import _dbus_bindings
 import dbus.glib
+import dbus.lowlevel
 import dbus.service
 
 from dbus._compat import is_py2, is_py3
@@ -81,6 +84,11 @@ NAME = "org.freedesktop.DBus.TestSuitePythonService"
 IFACE = "org.freedesktop.DBus.TestSuiteInterface"
 OBJECT = "/org/freedesktop/DBus/TestSuitePythonObject"
 
+# A random string that we should not transmit on the bus as a result of
+# the NO_REPLY flag
+SHOULD_NOT_HAPPEN = u'a1c04a41-cf98-4923-8487-ddaeeb3f02d1'
+
+
 class TestDBusBindings(unittest.TestCase):
     def setUp(self):
         self.bus = dbus.SessionBus()
@@ -197,6 +205,57 @@ class TestDBusBindings(unittest.TestCase):
         print("Delta: %f" % (b - a))
         self.assertTrue(True)
 
+    def testNoReply(self):
+        failures = []
+        report = []
+        main_loop = gobject.MainLoop()
+        unicode_type = (str if is_py3 else unicode)
+
+        def message_filter(conn, m):
+            print('Message filter received message: %r, %r' % (m, m.get_args_list()))
+
+            if conn is not self.bus:
+                failures.append('Message filter called on unexpected bus')
+
+            for a in m.get_args_list():
+                if isinstance(a, unicode_type):
+                    if SHOULD_NOT_HAPPEN in a:
+                        failures.append('Had an unexpected reply')
+                    elif a == 'TestNoReply report':
+                        report.extend(m.get_args_list())
+                        main_loop.quit()
+
+            return dbus.lowlevel.HANDLER_RESULT_NOT_YET_HANDLED
+
+        self.bus.add_message_filter(message_filter)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(True)
+        message.append(False)
+        message.set_no_reply(True)
+        self.bus.send_message(message)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(False)
+        message.append(False)
+        message.set_no_reply(True)
+        self.bus.send_message(message)
+
+        message = dbus.lowlevel.MethodCallMessage(NAME, OBJECT, IFACE, 'TestNoReply')
+        message.append(True)
+        message.append(True)
+        self.bus.send_message(message)
+
+        main_loop.run()
+        self.bus.remove_message_filter(message_filter)
+
+        if failures:
+            self.assertTrue(False, failures)
+
+        self.assertEqual(report[0], 'TestNoReply report')
+        self.assertEqual(report[1], 1)
+        self.assertEqual(report[2], 1)
+
     def testAsyncCalls(self):
         #test sending python types and getting them back async
         print("\n********* Testing Async Calls ***********")
